--------------------------------------------------------------------
-- ISA-Controller
-- Created on Thu Mar 02 10:07:54 2006
--------------------------------------------------------------------

-- FSM controller for ISA bus interfacing

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity controller is
port(
	CLK:		in std_logic;
	RST:		in std_logic;
	AEN:		in std_logic;
	ALE:		in std_logic;
	ADDR:		in std_logic_vector (23 downto 0);
	IOR:		in std_logic;
	IOW:		in std_logic;
	DATA:		inout std_logic_vector (15 downto 0);
	R_MATCH:	in std_logic;
	W_MATCH:	in std_logic;

	DATA_IN:	in std_logic_vector (15 downto 0);
	LD_DATA:	out std_logic;
	DATA_OUT:	out std_logic_vector (15 downto 0);
	ST_DATA:	out std_logic;
	ADDR_OUT:	out std_logic_vector (23 downto 0)
);
end controller;

architecture fsm of controller is

    type states is (S0,S1,S3,S3a,S4,S4a);
    signal nextState, currentState: states;

begin	  
	
	-- Steuerung des Zustandsregisters
    state_reg: process(CLK, RST)
    begin
        if (RST = '1') then 
            currentState <= S0;
        elsif (CLK'event and CLK = '1') then 
            currentState <= nextState;
        end if;
    end process;

    fsm_logic: process(currentState, AEN, ALE, ADDR, IOR, IOW, DATA, DATA_IN, R_MATCH, W_MATCH )
    begin

      case currentState is 

--		when S0 =>
--			DATA <= "0000000000000000";
--
--			LD_DATA <= '0';
--			DATA_OUT = "0000000000000000";
--			ST_DATA <= '0';
--			ADDR_OUT = "000000000000000000000000";
--			
--			nextState <= S1;
--		
--		when S1 =>
--			DATA_OUT <= DATA;
--
--			LD_DATA <= '0';
--			DATA_OUT = "0000000000000000";
--			ST_DATA <= '0';
--			ADDR_OUT = "000000000000000000000000";
--			
--			nextState <= S0;

        when S0 =>		-- wenn kein AEN, auf ALE warten und A23..17 latchen

        	DATA <= "ZZZZZZZZZZZZZZZZ";
			DATA_OUT <= "0000000000000000";
			ST_DATA <= '0';
			LD_DATA <= '0';
        	ADDR_OUT(23 downto 17) <= ADDR(23 downto 17);

			if (ALE='1' and AEN='0') then
				nextState <= S1;
			else
				nextState <= S0;
			end if;

		when S1 =>			-- anschliessend A16..0 latchen, entscheiden ob schreiben oder lesen

        	DATA <= "ZZZZZZZZZZZZZZZZ";
			DATA_OUT <= "0000000000000000";
			ST_DATA <= '0';
			LD_DATA <= '0';
			ADDR_OUT(16 downto 0) <= ADDR(16 downto 0);

			if (W_MATCH='1') then	-- je nach Adresse nach read oder write verzweigen
				nextState <= S3;
			elsif (R_MATCH='1') then
			 	nextState <= S4;
			else
				nextState <= S1;
			end if;

		when S3 =>				-- write operation (1/2)

        	DATA <= "ZZZZZZZZZZZZZZZZ";
			ST_DATA <= '0';
			LD_DATA <= '0';

			if (IOW='0') then 	-- ready to Write
				nextState <= S3a;
			else
				nextState <= S3;
			end if;
			
		when S3a =>				-- write operation (2/2)

        	DATA <= "ZZZZZZZZZZZZZZZZ";
		 	DATA_OUT <= DATA;	-- do write operation, then go back
			ST_DATA <= '1';		
			LD_DATA <= '0';
			
			nextState <= S0;
					
        when S4 =>				-- read operation (1/2)

        	DATA <= "ZZZZZZZZZZZZZZZZ";
			ST_DATA <= '0';
			LD_DATA <= '1';

			if (IOR='0') then	-- ready to Read
				nextState <= S4a;
			else
				nextState <= S4;
			end if;
		
		when S4a =>				-- read operation (2/2)
			DATA <= DATA_IN;	-- read operation durchfueren und zurueck zu S0
			ST_DATA <= '0';
			LD_DATA <= '1';

			nextState <= S0;
		
		when others =>			-- unerwarteter Zustand loest Reset aus
        	DATA <= "ZZZZZZZZZZZZZZZZ";
			ST_DATA <= '0';
			LD_DATA <= '0';

			nextState <= S0;

      end case;

    end process;

end fsm;

-- RW-Control --------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity rw_control is
port(	ADDR_IN: 	in std_logic_vector(23 downto 0);
        R_MATCH:	out std_logic;
        W_MATCH:	out std_logic
);
end rw_control;

architecture behv of rw_control is
begin				   
  
    process(ADDR_IN)
    begin
	    if (ADDR_IN="1010000000000000") then -- 0xA000 write
			W_MATCH <= '1';
			R_MATCH <= '0';
	    elsif (ADDR_IN="1010000000000001") then -- 0xA001 read
			W_MATCH <= '0';
			R_MATCH <= '1';	   
	    else
			W_MATCH <= '0';
			R_MATCH <= '0';
	    end if;		  
    end process;

end behv;

------------------------------------------------------------------
-- ISA-Controller
------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity isa_controller IS
	-- {{ALTERA_IO_BEGIN}} DO NOT REMOVE THIS LINE!
	port
	(
		-- PC-BUS-Signals
		B_CLK		: in std_logic;							-- PC: BUS TAKT
		B_RST		: in std_logic;							-- PC: RST
		B_AEN		: in std_logic;							-- PC: DMA IST AKTIV
		B_ALE		: in std_logic;							-- PC: LATCH FUER A17..23
		B_ADDR		: in std_logic_vector (23 downto 0);	-- PC: ADRESSBUS
		B_IOR		: in std_logic;							-- PC: IO READ
		B_IOW		: in std_logic;							-- PC: IO WRITE
		B_DATA		: inout std_logic_vector (15 downto 0);	-- PC: DATENBUS

		-- ISA-Controller generated Signals
		DATAIN		: in std_logic_vector (15 downto 0);	-- DATA IN
		DATALD		: out std_logic;						-- LOAD DATA
		DATAOUT		: out std_logic_vector (15 downto 0);	-- DATA OUT
		DATAST		: out std_logic;						-- STORE DATA
		ADDROUT		: out std_logic_vector (23 downto 0)	-- PC: ADRESSBUS
	);
	-- {{ALTERA_IO_END}} DO NOT REMOVE THIS LINE!
	
end isa_controller;

architecture isa_controller_arch of isa_controller is

component controller is	   
port(
	CLK:		in std_logic;
	RST:		in std_logic;
	AEN:		in std_logic;
	ALE:		in std_logic;
	ADDR:		in std_logic_vector (23 downto 0);
	IOR:		in std_logic;
	IOW:		in std_logic;
	DATA:		inout std_logic_vector (15 downto 0);
	R_MATCH:	in std_logic;
	W_MATCH:	in std_logic;

	DATA_IN:	in std_logic_vector (15 downto 0);
	LD_DATA:	out std_logic;
	DATA_OUT:	out std_logic_vector (15 downto 0);
	ST_DATA:	out std_logic;
	ADDR_OUT:	out std_logic_vector (23 downto 0)
);
end component;

component rw_control is
port(	ADDR_IN: 	in std_logic_vector(23 downto 0);
        R_MATCH:	out std_logic;
        W_MATCH:	out std_logic
);
end component;

signal ADDR_sig: std_logic_vector (23 downto 0);
signal WR_sig, RD_sig: std_logic;
signal Ald_sig, Dld_sig, Dst_sig: std_logic;

begin

    U0: controller port map(CLK => B_CLK,
							RST => B_RST,
							AEN => B_AEN,
							ALE => B_ALE,
							ADDR => B_ADDR,
							IOR => B_IOR,
							IOW => B_IOW,
							DATA => B_DATA,
							R_MATCH => RD_sig,
							W_MATCH => WR_sig,
							DATA_IN => DATAIN,
							LD_DATA => DATALD,
							DATA_OUT => DATAOUT,
							ST_DATA => DATAST,
							ADDR_OUT => ADDR_sig);
    U1: rw_control port map(ADDR_IN => ADDR_sig,
							R_MATCH => WR_sig,
							W_MATCH => RD_sig);

	-- ADDROUT <= ADDR_sig;

end isa_controller_arch;
